@@ -14,6 +14,7 @@ from typing import List
 
 from minikerberos import logger
 from minikerberos.common.ccache import CCACHE
+from minikerberos.common.spn import KerberosSPN
 from minikerberos.protocol.asn1_structs import METHOD_DATA, ETYPE_INFO, ETYPE_INFO2, \
 	PADATA_TYPE, PA_PAC_REQUEST, PA_ENC_TS_ENC, EncryptedData, krb5_pvno, KDC_REQ_BODY, \
 	AS_REQ, TGS_REP, KDCOptions, PrincipalName, EncASRepPart, EncTGSRepPart, PrincipalName, Realm, \
@@ -65,32 +66,37 @@ class AIOKerberosClient:
 		kc.kerberos_cipher = _enctype_table[kc.kerberos_cipher_type]
 		return kc
 
-	def build_asreq_lts(self, supported_encryption_method, kdcopts:List[str] = ['forwardable','renewable','proxiable']) -> AS_REQ:
+	def build_asreq_lts(self, supported_encryption_method, kdcopts:List[str] = ['forwardable','renewable','proxiable'], enctimestamp=None, newnow=None, no_preauth = False, kdc_req_body_extra = None) -> AS_REQ:
 		logger.debug('Constructing TGT request with auth data')
 		#now to create an AS_REQ with encrypted timestamp for authentication
+		padatas = []
 		pa_data_1 = {}
 		pa_data_1['padata-type'] = int(PADATA_TYPE('PA-PAC-REQUEST'))
 		pa_data_1['padata-value'] = PA_PAC_REQUEST({'include-pac': True}).dump()
-		
-		now = datetime.datetime.now(datetime.timezone.utc)
-		#creating timestamp asn1
-		timestamp = PA_ENC_TS_ENC({'patimestamp': now.replace(microsecond=0), 'pausec': now.microsecond}).dump()
-		
+		padatas.append(pa_data_1)
 		
 		logger.debug('Selecting common encryption type: %s' % supported_encryption_method.name)
-		self.kerberos_cipher = _enctype_table[supported_encryption_method.value]
-		self.kerberos_cipher_type = supported_encryption_method.value
-		self.kerberos_key = Key(self.kerberos_cipher.enctype, self.usercreds.get_key_for_enctype(supported_encryption_method, salt = self.server_salt))
-		enc_timestamp = self.kerberos_cipher.encrypt(self.kerberos_key, 1, timestamp, None)
-		
-		
-		pa_data_2 = {}
-		pa_data_2['padata-type'] = int(PADATA_TYPE('ENC-TIMESTAMP'))
-		pa_data_2['padata-value'] = EncryptedData({'etype': supported_encryption_method.value, 'cipher': enc_timestamp}).dump()
-		
+		now = datetime.datetime.now(datetime.timezone.utc)
+		if no_preauth is False:
+			if enctimestamp is None:
+				#creating timestamp asn1
+				timestamp = PA_ENC_TS_ENC({'patimestamp': now.replace(microsecond=0), 'pausec': now.microsecond}).dump()
+				self.kerberos_cipher = _enctype_table[supported_encryption_method.value]
+				self.kerberos_cipher_type = supported_encryption_method.value
+				self.kerberos_key = Key(self.kerberos_cipher.enctype, self.usercreds.get_key_for_enctype(supported_encryption_method, salt = self.server_salt))
+				enc_timestamp = self.kerberos_cipher.encrypt(self.kerberos_key, 1, timestamp, None)
+			else:
+				now = newnow
+				enc_timestamp = enctimestamp
+			
+			pa_data_2 = {}
+			pa_data_2['padata-type'] = int(PADATA_TYPE('ENC-TIMESTAMP'))
+			pa_data_2['padata-value'] = EncryptedData({'etype': supported_encryption_method.value, 'cipher': enc_timestamp}).dump()
+			padatas.append(pa_data_2)
+
 		kdc_req_body = {}
 		kdc_req_body['kdc-options'] = KDCOptions(set(kdcopts))
-		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body['realm'] = self.usercreds.domain.upper()
 		kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
 		kdc_req_body['till'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -98,10 +104,14 @@ class AIOKerberosClient:
 		kdc_req_body['nonce'] = secrets.randbits(31)
 		kdc_req_body['etype'] = [supported_encryption_method.value] #selecting according to server's preferences
 		
+		if kdc_req_body_extra is not None:
+			for key in kdc_req_body_extra:
+				kdc_req_body[key] = kdc_req_body_extra[key]
+		
 		kdc_req = {}
 		kdc_req['pvno'] = krb5_pvno
 		kdc_req['msg-type'] = MESSAGE_TYPE.KRB_AS_REQ.value
-		kdc_req['padata'] = [pa_data_2,pa_data_1]
+		kdc_req['padata'] = padatas
 		kdc_req['req-body'] = KDC_REQ_BODY(kdc_req_body)
 		
 		return AS_REQ(kdc_req)
@@ -117,7 +127,7 @@ class AIOKerberosClient:
 
 		kdc_req_body_data = {}
 		kdc_req_body_data['kdc-options'] = KDCOptions(set(kdcopts))
-		kdc_req_body_data['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body_data['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body_data['realm'] = self.usercreds.domain.upper()
 		kdc_req_body_data['sname'] = PrincipalName({'name-type': NAME_TYPE.SRV_INST.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
 		kdc_req_body_data['till']  = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -199,7 +209,7 @@ class AIOKerberosClient:
 			for tgt, keystruct in self.ccache.get_all_tgt():
 				if self.usercreds.ccache_spn_strict_check is True:
 					our_user = str(self.usercreds.username) + '@' + self.usercreds.domain
-					ticket_for = tgt['cname']['name-string'][0] + '@' + tgt['crealm']
+					ticket_for = '/'.join(tgt['cname']['name-string']) + '@' + tgt['crealm']
 					if ticket_for.upper() == our_user.upper():
 						logger.debug('Found TGT for user %s' % our_user)
 						self.kerberos_TGT = tgt
@@ -228,7 +238,7 @@ class AIOKerberosClient:
 		#now getting server's supported encryption methods
 		
 		supp_enc_methods = collections.OrderedDict()
-		for enc_method in METHOD_DATA.load(rep['e-data']).native:					
+		for enc_method in METHOD_DATA.load(rep['e-data']).native:
 			data_type = PaDataType(enc_method['padata-type'])
 			
 			if data_type == PaDataType.ETYPE_INFO or data_type == PaDataType.ETYPE_INFO2:
@@ -252,7 +262,7 @@ class AIOKerberosClient:
 		return preferred_enc_type
 
 
-	async def get_TGT(self, override_etype = None, decrypt_tgt = True, kdcopts = ['forwardable','renewable','proxiable']):
+	async def get_TGT(self, override_etype = None, decrypt_tgt = True, kdcopts = ['forwardable','renewable','proxiable'], override_sname:KerberosSPN = None):
 		"""
 		decrypt_tgt: used for asreproast attacks
 		Steps performed:
@@ -270,9 +280,13 @@ class AIOKerberosClient:
 		now = datetime.datetime.now(datetime.timezone.utc)
 		kdc_req_body = {}
 		kdc_req_body['kdc-options'] = KDCOptions(set(kdcopts))
-		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body['realm'] = self.usercreds.domain.upper()
-		kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
+		if override_sname is None:
+			kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
+		else:
+			# if we want to directly kerberoast with no-preauth user
+			kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.SRV_INST.value, 'name-string': override_sname.get_principalname()})
 		kdc_req_body['till']  = (now + datetime.timedelta(days=1)).replace(microsecond=0)
 		kdc_req_body['rtime'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
 		kdc_req_body['nonce'] = secrets.randbits(31)
@@ -298,16 +312,17 @@ class AIOKerberosClient:
 				
 		if rep.name != 'KRB_ERROR':
 			#user can do kerberos auth without preauthentication!
-			self.kerberos_TGT = rep.native
+			rep = rep.native
+			self.kerberos_TGT = rep
 
 			#if we want to roast the asrep (tgt rep) part then we dont even have the proper keys to decrypt
 			#so we just return, the asrep can be extracted from this object anyhow
-			if decrypt_tgt == False:
-				return
+			if decrypt_tgt == False or self.usercreds.nopreauth is True:
+				return rep
 
-			self.kerberos_cipher = _enctype_table[23]
-			self.kerberos_cipher_type = 23
-			self.kerberos_key = Key(self.kerberos_cipher.enctype, self.usercreds.get_key_for_enctype(EncryptionType.ARCFOUR_HMAC_MD5))
+			self.kerberos_cipher = _enctype_table[rep['enc-part']['etype']]
+			self.kerberos_cipher_type = rep['enc-part']['etype']
+			self.kerberos_key = Key(self.kerberos_cipher.enctype, self.usercreds.get_key_for_enctype(EncryptionType(rep['enc-part']['etype'])))
 			
 		else:
 			if rep.native['error-code'] != KerberosErrorCode.KDC_ERR_PREAUTH_REQUIRED.value:
@@ -334,7 +349,6 @@ class AIOKerberosClient:
 		else:
 			cipherText = rep['enc-part']['cipher']
 			temp = self.kerberos_cipher.decrypt(self.kerberos_key, 3, cipherText)
-		
 			try:
 				self.kerberos_TGT_encpart = EncASRepPart.load(temp).native
 			except Exception as e:
@@ -344,14 +358,14 @@ class AIOKerberosClient:
 				except Exception as e:
 					logger.error('Failed to load decrypted part of the reply!')
 					raise e
-					
+			
 			self.kerberos_session_key = Key(self.kerberos_cipher.enctype, self.kerberos_TGT_encpart['key']['keyvalue'])
 			self.ccache.add_tgt(self.kerberos_TGT, self.kerberos_TGT_encpart, override_pp = True)
 			logger.debug('Got valid TGT')
 			
 			return 
 
-	def tgs_from_ccache(self, spn_user, override_etype):
+	def tgs_from_ccache(self, spn_user:KerberosSPN, override_etype):
 		try:
 			if self.ccache is None:
 				raise Exception('No CCACHE file found')
@@ -377,7 +391,7 @@ class AIOKerberosClient:
 		except Exception as e:
 			return None, None, None, e
 
-	async def get_TGS(self, spn_user, override_etype = None, is_linux = False):
+	async def get_TGS(self, spn_user:KerberosSPN, override_etype = None, is_linux = False):
 		"""
 		Requests a TGS ticket for the specified user.
 		Retruns the TGS ticket, end the decrpyted encTGSRepPart.
@@ -410,7 +424,12 @@ class AIOKerberosClient:
 		if override_etype:
 			kdc_req_body['etype'] = override_etype
 		else:
-			kdc_req_body['etype'] = [self.kerberos_cipher_type]
+			if self.kerberos_cipher_type == -128:
+				# we dunno how to do GSS api calls with -128 etype,
+				# but we can request etype 23 here for which all is implemented
+				kdc_req_body['etype'] = [23]
+			else:
+				kdc_req_body['etype'] = [self.kerberos_cipher_type]
 
 		authenticator_data = {}
 		authenticator_data['authenticator-vno'] = krb5_pvno
@@ -499,7 +518,7 @@ class AIOKerberosClient:
 		
 		krb_tgs_body = {}
 		krb_tgs_body['kdc-options'] = KDCOptions(set(kdcopts))
-		krb_tgs_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		krb_tgs_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		krb_tgs_body['realm'] = self.usercreds.domain.upper()
 		krb_tgs_body['till'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
 		krb_tgs_body['nonce'] = secrets.randbits(31)
@@ -607,7 +626,6 @@ class AIOKerberosClient:
 		pa_for_user['padata-value'] = PA_FOR_USER_ENC(pa_for_user_enc).dump()
 	
 		###### Constructing body
-		spn_user = [self.usercreds.username]
 		if spn_user is not None:
 			if isinstance(spn_user, str):
 				spn_user = [spn_user]
@@ -615,6 +633,8 @@ class AIOKerberosClient:
 				spn_user = spn_user
 			else:
 				spn_user = spn_user.get_principalname()
+		else:
+			spn_user = self.usercreds.username.split('/')
 
 		
 		krb_tgs_body = {}
