@@ -84,7 +84,7 @@ class KerbrosClient:
 		
 		kdc_req_body = {}
 		kdc_req_body['kdc-options'] = KDCOptions(set(kdcopts))
-		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body['realm'] = self.usercreds.domain.upper()
 		kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
 		kdc_req_body['till'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -111,7 +111,7 @@ class KerbrosClient:
 
 		kdc_req_body_data = {}
 		kdc_req_body_data['kdc-options'] = KDCOptions(set(kdcopts))
-		kdc_req_body_data['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body_data['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body_data['realm'] = self.usercreds.domain.upper()
 		kdc_req_body_data['sname'] = PrincipalName({'name-type': NAME_TYPE.SRV_INST.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
 		kdc_req_body_data['till']  = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -221,7 +221,7 @@ class KerbrosClient:
 		now = datetime.datetime.now(datetime.timezone.utc)
 		kdc_req_body = {}
 		kdc_req_body['kdc-options'] = KDCOptions(set(['forwardable','renewable','proxiable']))
-		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		kdc_req_body['cname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		kdc_req_body['realm'] = self.usercreds.domain.upper()
 		kdc_req_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': ['krbtgt', self.usercreds.domain.upper()]})
 		kdc_req_body['till']  = (now + datetime.timedelta(days=1)).replace(microsecond=0)
@@ -445,7 +445,6 @@ class KerbrosClient:
 		pa_for_user['padata-value'] = PA_FOR_USER_ENC(pa_for_user_enc).dump()
 	
 		###### Constructing body
-		spn_user = [self.usercreds.username]
 		if spn_user is not None:
 			if isinstance(spn_user, str):
 				spn_user = [spn_user]
@@ -453,6 +452,8 @@ class KerbrosClient:
 				spn_user = spn_user
 			else:
 				spn_user = spn_user.get_principalname()
+		else:
+			spn_user = self.usercreds.username.split('/')
 
 		krb_tgs_body = {}
 		krb_tgs_body['kdc-options'] = KDCOptions(set(kdcopts))
@@ -751,7 +752,7 @@ class KerbrosClient:
 		
 		krb_tgs_body = {}
 		krb_tgs_body['kdc-options'] = KDCOptions(set(kdcopts))
-		krb_tgs_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': [self.usercreds.username]})
+		krb_tgs_body['sname'] = PrincipalName({'name-type': NAME_TYPE.PRINCIPAL.value, 'name-string': self.usercreds.username.split('/')})
 		krb_tgs_body['realm'] = self.usercreds.domain.upper()
 		krb_tgs_body['till'] = (now + datetime.timedelta(days=1)).replace(microsecond=0)
 		krb_tgs_body['nonce'] = secrets.randbits(31)
